// (C) Copyright 1993, 94 by Heiko Damerow MPG AG "Roentgenbeugung"
// Datei m_data.h
//! Erweiterung, Kommentierung Bernhard Buss, IfI

#ifndef __M_DATA_H
#define __M_DATA_H


typedef struct
{
    float omega,theta,intens;
}
TBMContens;


//! Bildschrim hat x und y Ausdehnung
typedef struct
{
    int x0, x1, dx, y0, y1, dy;
}
TScreen;

//! Koordinatensystem mit x,y,z
//! und jeweils einer Angabe 1 Pixel in x,y,z Richtung
//! entspricht *SF Koordinatensystemeinheit
//! und eingestellter Ausgabetyp
typedef struct
{
    double xMin, xMax, xSF;
    double yMin, yMax, ySF;
    double zMin, zMax, zSF;
    TScaleType xST, yST, zST;
}
TCoorSystem;

//! Aufzaezhlung Ausgabetypen/Darstellungsformen
typedef enum
{
    ReciprokeLatticeBitmap = 1 , MatrixBitmap = 2,
    Curve = 3 //! weil sowieso nicht implementiert, Number, BarGraph, Histogram
}
TOutputType;

//! Umrechnung Alt-/Neugrad...
#define DegToRad(val) ((val) * 0.01745329251994)
#define RadToDeg(val) ((val) / 0.01745329251994)

//! Umrechnung in Reziproke Einheiten
//! gendert, weil Umrechnung so korrekt und Darstellung nicht gespiegelt...
#define ArToRLx(_ome, _the) ((cos((_the)-(_ome))- cos(_ome)))
//!#define ArToRLx(_ome, _the) ((cos(_ome) - cos((_the) - (_ome))))
#define ArToRLy(_ome, _the) ((sin((_the)-(_ome))+ sin(_ome)))
//!#define ArToRLy(_ome, _the) ((sin(_ome) + sin((_the) - (_ome))))

//! Klasse um Bitmaps aus Kurvendaten zu errechnen und darzustellen
class TBitmapSource
{
    //! leider alles friend!, was in m_data.h
    friend class TPlotData;
    friend class TCurveShowParam;
    friend class TCurveFreeScal;
    friend class TCurveFreeScalColor;

public:

    //! Konstruktor
    TBitmapSource( TPlotData * );

    //! Destruktor
    virtual ~TBitmapSource();

    //! wird aufgerufen, wenn ein Bitmap nachgeladen werden soll (m_arscan.cpp)
    BOOL ProcessBitmapFile( HFILE );

    //! generiert den Header fuer ein Bitmap, setzt Koordinatensystem und Screen
    void FormatDBaseToBitmapSource( void );

    //! nicht implementiert
    void UpdateBitmapSource( void );

    //! False - keine Graustufendarstellung, True - Graustufendarstellung in Bitmaps
    //! wird im ini-File mit "swPalette" gesetzt
    BOOL bGrayScalePalette;

    //! Zeiger auf "Daten"feld
    //! hier wird zu jeder [x,y]-Pixelkoordinate der zugehoerigen
    //! Theta-/OmegaWert gespeichert (nur RL-Bitmap)
    TBMContens** daten;

protected:

    void New( void );

    //! laedt Bitmap Ressource - benoetigt?
    void LoadBitmap( HDC );

    //! berechnet Bytes pro Zeile*Hoehe des Bitmaps - Gesamtgroesse im Speicher
    DWORD GetImageSize( void );

    //! ruft bei RL_Bitmap DrawMeasurementArea()
    //! berechnet Position der Anordnung der Farblegende
    //! setzt Min,MaxIntensitaeten
    //! ruft DrawColorTable()
    void DrawBitmapFrame( HDC );

    //! setzt Screen-Variablen, entsprechend der Bildschirmgroesse
    //! setzt Units, Orientation, Scaling
    void SetScreen( HDC, TScreen& );

    //! berechnet entsprechende Intensitten
    //! und gibt die Legende zu den Intensitten aus
    void DrawColorTable( HDC, TColorTable& );

    //! falls RL-Bitmapdarstellung, berechnet und zeichnet
    //! das untersuchte Messgebiet im den reziproken Raum
    void DrawMeasurementArea( HDC );

    //! fuellt den Header eines Bitmaps mit Farbwerten der Palette
    void FillBMInfoFromPalette( LOGPALETTE* );

    //! Bitblock-Transfer von hMemDC (dem gegebenen Quell-DC)
    //! zu diesem HDC (dem Ziel-DC) - entspricht der Darstellung eines Bitmaps
    void DrawBitmap( HDC, int, int );

    BITMAPFILEHEADER BMPFileHdr;

    //! Zeiger auf ColorTable-Objekt
    TColorTable ColorTable;

    //! Zeiger auf zugehoeriges TPlotData-Objekt
    TPlotData *DataWnd;

    //! Zeiger auf zugehoerigen TCurveFreeScalColor-Dialog
    TCurveFreeScalColor *UserFarb;

private:

    //! function creates a device-dependent bitmap (DDB) from a device-independent
    //! bitmap (DIB) and, optionally, sets the bitmap bits
    HBITMAP CreateGDIObject( HDC );

    //! Zwischenablageoperation, wenn Typ CF_DIB
    HGLOBAL RenderDIB( void );

    //! ruft CreatePaletteFromDIB(), ansonsten CreateDefaultPalette()
    LOGPALETTE * GetPalette( void );

    //! generiert Palette von DIB (Device independend Bitmap)
    LOGPALETTE * CreatePaletteFromDIB( void );

    //! generiert neue Palette (Graustufen oder Bunt)
    LOGPALETTE * CreateDefaultPalette( void );

    //! gibt Darstellungtyp zurueck
    //! wird nicht benutzt
    // TOutputType GetType( void );

    //! um Intensitaetsabstufungen im User Falle zu generieren
    //! ansonsten werden Defaultwerte gesetzt
    //! fuellt ColorTable.ColorMap[cnt] mit Werten
    void ColorDef(void);

    //! setzt Variablenfelder farbe[] und ber[]
    //! so dass ein linearer Verlauf abhaengig von Min-,MaxIntensitaet
    void BereichDef(void);

    //! gibt zu uebergebener Intensitaet entsprechende Intensitaetsfarbe zurueck
    int GetColor( float );

    //! generiere MatrixBitmap
    void GenerateAngleSpaceBitmap(void);

    //! genriere RL-Bitmap
    void GenerateRLBitmap( void );

    //! eigenes Koordinatensystem fuer Bitmapberechnung
    TCoorSystem* BMCoorSys;

    //! wichtig fuer Bitmap-Generierung
    HGLOBAL hDIBInfo;

    HGLOBAL hDIBData;

    HGLOBAL hMemPal;

    HPALETTE hPalette, hOldPalette;

    HBITMAP hBitmap;

    //! nicht mehr benoetigt, da GenerateAngleSpaceBitmap() geaendert
    //! int nEnlargedLines;

    //! int nAddedPoints;

    //! Feld mit Intensitaetsabstufungen im Falle von "User Skalierung"
    float ber[8];

    //! Feld mit Farbabstufungen im Falle von "User Skalierung"
    int farbe[8];

    //! TRUE - "Daten"feld existiert, FALSE - "Daten"feld existiert nicht
    //! (bei RL-Bitmap wird zu jedem Bildpunkt Theta/Omega gespeichert)
    BOOL dateniscreated;

    //! Speicherung der x-Bildpunkte, y-Bildpunkte
    //! zwecks Vergleich, wenn neues "Daten"feld angelegt werden muss
    int dxold,dyold;

    //! Speicherung des alten Filenamens
    //! zwecks Vergleich, wenn neues "Daten"feld angelegt werden muss
    char Fileold[MaxString];
};



// Stellt die Funktionalitaet der Darstellung von Kurven und der Bewertung
// dieser Kurven bereit
class TPlotData : public TMDIWindow
{
    friend class TCurveShowParam;
    friend class TCurveFreeScal;
    friend class TBitmapSource;
    friend class TCurveFreeScalColor;

public:

    //! Konstruktor
    TPlotData( void );

    //! Destruktor
    virtual ~TPlotData( void );

    // void Size( HWND, WPARAM, LPARAM );

    //! fllt die Struktur lpPoints mit den Werten fr eine Kurve
    //! um damit spter dann die Kurve darzustellen
    void FormatCurveToPLine( TCurve *, LPPOINT, int & );

    //! Paint wird von EvPaint aufgerufen und automatisch von Windows angefordert,
    //! um den Inhalt des Fensters neu anzuzeigen
    void Paint( HDC, PAINTSTRUCT * );

    //! behandelt Kommando: linke Maustaste wurde gedrueckt
    virtual void lButtonDown( WPARAM, LPARAM );

    //! wenn linke Maustaste gedrueckt gehalten und dabei bewegt wird,
    //! wird der Abstand zur Pos. des ersten Mausklicks berechnet
    virtual void MouseMove( WPARAM, LPARAM );

    //! gibt Handle zum zeichnen im Kindfenster zurck + linke Maustaste
    //! als nicht gedrueckt kennzeichnen
    virtual void lButtonUp( WPARAM, LPARAM );

    //! wenn Darstellungstyp ein Bitmap, dann bereite Zwischenablage vor
    void RenderFormat( WPARAM );

    //! raeumt Zwischenablage auf, wenn Besitzer der Zwischenablage tot
    void RenderAllFormats( WPARAM );

    virtual BOOL New( void );

    //! sichert Daten in der SecondCurve
    //! m_main.cpp im Falle von cm_MoveScanReady
    void PickUpData( LPARAM );

    //! sichert Koordinatensystem in AltCoorsys
    //! m_main.cpp im Falle von cm_MoveScanReady
    void SetKSProperties( TKSProperties* );

    //! fixiert die aktuelle Hauptkurve -> setzt Pointer von SecondCurve auf
    //! MainCurve
    void FreezeCurve( void );

    //! setzt SecondCurve auf Initialwerte zurueck (loeschen aller Metripel)
    //! zeichnet Scan-Fenster neu
    void KillSecondCurve( void );

    //! speichert fixierte Kurve (SecondCurve) unter auszuwaehlendem Dateinamen
    //! mit Extension *.bk
    void SaveSecondCurve( void );

    //! setzt Variable bEnablePaint
    void EnablePaint( void )
    {
        bEnablePaint = TRUE;
    };

    //! setzt Variable bEnablePaint
    void DisablePaint( void )
    {
        bEnablePaint = FALSE;
    };

protected:

    //! setzt den Filetyp, nach dem Laden von alten Daten
    void SetFileType(int);

    //! Funktion, die die bentigte Nachkommastellengenauigkeit
    //! berechnet und die ntigen Nachkommastellen zurckliefert
    int GetNachkommaStelle(TCoorSystem* pCoorSys,int x,int y);

    //! zeichnet Koord.System
    void DrawCoordinateSystem( HDC );

    //! laedt die Cursor-Ressource in den Speicher und vergibt ein Handle
    HCURSOR GetCursor( void )
    {
        return LoadCursor( NULL, IDC_CROSS );
    };

    //! Kopieren mit Hilfe der Zwischeablage - Typen: CF_DIB, CF_PALETTE
    void DoCopy( void );

    //! fuelle TKSystem mit Standardwerten
    virtual BOOL SetMeasurementArea( TKSystem & );

    //! setze neue Koordinatensystembereiche
    virtual void SetRanges( void );

    //! macht nix...
    virtual double GetThetaOffset( int )
    {
        return 0.0;
    };

    HFONT hFont;

    //! Zeiger auf zugehoeries Bitmap
    TBitmapSource* BSource;

    TCurve* MainCurve;

    TCurve* SecondCurve;

    //! gibt es ein 2. Koordinatensystem
    BOOL bSecondaryCoor;

    //! XY-Skalierung an/aus
    BOOL bXY_Scaling;

    //! gibt Darstellungstyp an
    TOutputType eOutputType;

    //! MinIntensitaet, MaxIntensitaet
    float LowerBound, UpperBound;

    //! erstes und alternatives Koordinatensystem
    TCoorSystem CoorSys, AltCoorSys;

    TScreen Screen;

    //! existiert Bitmap?
    BOOL bBitmapIsCreated;

    //! beiinhaltet den geladenen FileTyp
    int eLoadFormat;

private:

    //! Darstellung zulassen oder nicht
    BOOL bEnablePaint;

    //! Mousebutton unten/oben
    BOOL bButtonDown;

    //! Koordinaten wohin mit der Maus geklickt wurde
    float xo, yo;

    //! beinhaltet Handle auf ein DC (WinAPI)
    HDC DragDC;
};

//! generiert Modaldialog "Einstellungen zur Darstellung"
class TCurveShowParam : public TModalDlg
{
public:
    TCurveShowParam(TMDIWindow *);
protected:

    //! Funktionen die den Dialog steuern...
    void Dlg_OnCommand( HWND, int, HWND, UINT );
    BOOL Dlg_OnInit( HWND, HWND, LPARAM );
    BOOL CanClose( void );
    BOOL LeaveDialog( void );
    void Dlg_OnHScrollBar(HWND ,HWND ,UINT ,int );

private:

    //! den Dialog kann man uebr den Button "User Skalierung" aufrufen
    TModalDlg *FreeScaldlg;

    //! Groesse der y-Achse ueber Scrollbalken beeinflussbar
    int scrolldypos;

    //! Groesse der x-Achse ueber Scrollbalken beeinflussbar
    int scrolldxpos;

    //! fast alle Plotdaten werden gebraucht
    TPlotData *PlotWindow;

    //! eigenes Koordinatensystem/Alternatives
    TCoorSystem *CoorSys,*AltCoorSys;

    //! nicht mehr benoetigt
    // TOutputType bOutputToShow;

    //! einige Handles behalten...
    HWND hWndOutputT, hWndScaleT,hWndScrolldy,hWndScrolldx;
};


//! neuer Modaldialog fuer freie Intensitaetskalierung
class TCurveFreeScal : public TModalDlg
{
    friend class TCurveFreeScalColor;
public:
    TCurveFreeScal( TMDIWindow * );
protected:

    //! Funktionen die den Dialog steuern...
    void Dlg_OnCommand( HWND, int, HWND, UINT );
    BOOL Dlg_OnInit( HWND, HWND, LPARAM );
    BOOL CanClose( void );
    BOOL LeaveDialog( void );

private:

    //! zum Speichern der RessourcenIDs
    int FarbeId[8];
    int BereichId[8];
    //! Plotdaten werden gebraucht
    TPlotData* PlotWindow;
};

class TCurveFreeScalColor : public TMDIWindow
{
public:

    //! Konstruktor
    TCurveFreeScalColor(TMDIWindow*);

    //! Destruktor
    ~TCurveFreeScalColor(void);

    //! Titel setzen
    virtual BOOL SetTitle();

    //! Fenster kreieren (ueberladene Funktion)
    virtual void Create(HWND hwnd);

    //! eigene Paint-Funktion (ueberladene Funktion) WinAPI
    void Paint(HDC,PAINTSTRUCT *);
protected:

    //! zeichne Farblegende
    void DrawColorTable(HDC);

    //! Zeiger auf TPlotData-Objekt
    TPlotData* PlotWindow;
};


#endif
// __M_DATA_H
// __LastLine__

