//## Kommentare dieser Form (mit zwei Doppelkreuzen) sind von Marlies Gollnick ~:)
#include    "rc_def.h"
#include    "comhead.h"
#pragma hdrstop

// m_topo.cpp
// (C) 1993,1994 by Heiko Damerow MPG AG "Rntgenbeugung"
// Modul Topographie
#include		"m_devcom.h"
#include		"m_layer.h"
#include    "m_steerg.h"
#include    "m_topo.h"

//## Formatierte Zeitausgabe
#define 		M_TimeString(buffer,TTicks) 											\
										(sprintf(	(LPSTR)buffer,"%02d:%02d:%02d", \
															(int)(TTicks/3600000l), 				\
															(int)(TTicks/60000l) % 60,			\
															(int)(TTicks/1000l) % 60));


/*
* Container-Klasse TTopography fr Topographie-Parameter
* Konstruktor TTopography
* Dialog zum Ausfhren der Topographie: TTopographyExecute
* Dialog zum Setzen der Parameter: TTopographySetParam
*/

extern  LPDList		lpDList;
extern  TSteering		Steering;

TTopography 	Topography;
TMacroTag* 		MWorkPoint;

//***** Parameter-Reprsentation fr die Topographie ***************************

//##Belegung einiger Variablen von TTopographie mit Initialwerten
TTopography::TTopography(void)
	{
	fMoveStep 				 = 0.3;
	fMaxAngleEscape       = 50;     // arcsec
	lMeasurementTime      = 600;  	// Sekunden
	nNumberCycle      	 = 1;    //##Anzahl der Mehrfachbelichtungen muss != 0
											//##sein, da sonst Topographie Einstellungsfenster
											//##nicht mit OK zu schliessen ist
	bMultibleShot         = FALSE;  //##Mehrfachbelichtung
	dStartAngle           = -20.0;  // arcsec
	fAngleBetweenShots    = 3.0;
	fFailure         	    = 0.02;  //##Messfehler des Counters wird nicht mehr
											 //##geaendert
	bMonitorUsed      	 = FALSE;
	Sensor            	 = NULL;
	Monitor               = NULL;
	nMotor					 = -1;
	};

//##Einlesen einiger Werte aus dem Ini-File
void TTopography::Initialize(void)
	{
	char	Ident[] = "Topography";
	char	buf[MaxString];

	GetPrivateProfileString(Ident,"WorkPoint","-60.0",buf,MaxString,GetCFile());
	fWorkPoint            = fabs(atof(buf) / 100.0);
	bSmallAngleSide       = (atof(buf) < 0);
	GetPrivateProfileString(Ident,"ControlRange","3.0",buf,MaxString,GetCFile());
	fControlRange         = atof(buf) / 100.0;
	GetPrivateProfileString(Ident,"ControlStep","0.2",buf,MaxString,GetCFile());
	fControlStep					= atof(buf);
	GetPrivateProfileString(Ident,"ExposureTime","10.0",buf,MaxString,GetCFile());
	fMaxTime  						= atof(buf);     //##Detektormesszeit
	GetPrivateProfileString(Ident,"ExposureCounts","10000",buf,MaxString,GetCFile());
	dwMaxCounts      	    = atol(buf);
	};

//*** Benutzer-Schnittstelle fr die Topographie *******************************
//******************************************************************************

//##Konstruktor des Topographie Ausfuehrungs Fensters
//##Belegung einiger Initialwerte (Zustandsvariablen, sowie Counter und
//##Motor, falls dies noch nicht geschehen ist)
TTopographyExecute::TTopographyExecute(void):TModalDlg("TopographyExecute")
	{
	//##Belegung einiger Zustandvariablen
	bStarted				= FALSE;
	bSetupOk 			= TRUE;   //##Motor und Counter sind durch Nutzer gesetzt
	bControlActive    = FALSE;
	bTimeRunning      = FALSE;
	bExeptionOccured  = FALSE;
	bAdditionalTime   = FALSE;
	//## wird TTopographyExecute ohne vorherige Ausfuehrung von
	//## TTopographySetParam zum ersten mal aufgerufen, so werden
	//## der Sensor auf den aktuellen Zaehler gesetzt und
	//## nMotor auf Omega
	if(Topography.Sensor == NULL)
		{
		Topography.Sensor  = lpDList->DP();
		Topography.Monitor = Device;
		Topography.bMonitorUsed = FALSE;
		bSetupOk = FALSE;
		}
	if(Topography.nMotor == -1)
		{
		//## als Motorenachse Omega setzen
		Topography.nMotor  = mlGetIdByName(Omega);
		bSetupOk = FALSE;
		}
	};

#pragma argsused
//##diese Funktion wird nach Empfang der Message WM_INITDIALOG aufgerufen
//##siehe DialogProc in dlg_tpl.cpp
//##weitere Initialisierung vor Darstellung des Topographie Ausfuehren Fensters
//##(Counter, Motor)
BOOL TTopographyExecute::Dlg_OnInit(HWND hwnd,HWND hwndCtl,LPARAM lParam)
	{
	char    buf[MaxString];
	// Detektor initialisieren
	Device = Topography.Sensor;
	nMotor = Topography.nMotor;
	//##falls zum Zaehler kein Fenster angezeigt wird, neues Zaehlerfenster
	//##oeffnen
	if(!Device->bDeviceOpen)
		{
		NewWindow((TCounterWindow *)new TCounterWindow(Device->GetId()));
		}
	// initialize motor
	//##setzt den aktiven Motor auf nMotor
	mlSetAxis(nMotor);
	//##weisst nMotor den aktiven Motor zu, nachdem zuvor der aktive Motor
	//##auf den Wert von nMotor gesetzt wurde -> eigentlich sinnlos
	nMotor = mlGetAxis();
	//## Werte des aktuellen Motors in allgemeinere Settings Struktur (m_motcom.h
	//## TMSettingS) eintragen
	mPushSettings();

	CurrentTime = StartTime = 0;

	//##Festlegung von Beschriftung und darzustellenden Werten fuer das
	//##Fenster
	M_TimeString (buf,CurrentTime);
	SetDlgItemText(GetHandle(),id_MeasurementTime,(LPSTR)buf);
	sprintf(buf,"%.2f",0.0);
	SetDlgItemText(GetHandle(),id_Angle,(LPSTR)buf);
	sprintf(buf,"%d",Topography.nNumberCycle);
	SetDlgItemText(GetHandle(),id_RemainderCyclus,(LPSTR)buf);
	RestShots = Topography.nNumberCycle;
	if(Topography.bMultibleShot)
		SetDlgItemText(GetHandle(),cm_SwitchControl,"&Topographie starten");
		else
		SetDlgItemText(GetHandle(),cm_SwitchControl,"&Regeln starten");
	// Ablaufsteuerung initialisieren
	//## Reportausgabe fuer Steering wird auf true gesetzt
	Steering.Visualising(1,0,0,0,0,0);
	//## Makro zur Arbeitspunkteinstellung holen und anpassen
	MWorkPoint = Steering.GetMacroByName("SetupTopography");
	if(Topography.bSmallAngleSide)
		MWorkPoint->CmdList[1].P1 = SmallSide;
		else
		MWorkPoint->CmdList[1].P1 = LargeSide;
	sprintf(MWorkPoint->CmdList[1].P3,mGetDF(),fabs(Topography.fWorkPoint));
	//##cm_Initialize und cm_ParamSet Message senden, und deren Abarbeitung
	//##abwarten
	SendMessage(GetHandle(),WM_COMMAND,cm_Initialize,0);
	SendMessage(GetHandle(),WM_COMMAND,cm_ParamSet,0);
	//##Zustandsvariablen setzen
	bStartPointOk = FALSE;
	bStarted = TRUE;
	return TRUE;
	};

//##Zeitueberpruefung fuer die Mehrfachbelichtung und
//##nach Messzeitende Ausgabe Beepton
void TTopographyExecute::Dlg_OnTimer(HWND hwnd,UINT id)
	{
	char    buf[MaxString];

	switch(id) {
		//## Gesamtmesszeit war abgelaufen; Ausgabe eines regelmaessigen Beeptons
		case 99:
		MessageBeep(0);
		DelayTime(100);
		MessageBeep(0);
		return;

		//##Zeitueberpruefung waehrend der Mehrfachbelichtung
		case 88:  // Timer fr Mehrfach-Belichtung
		FORWARD_WM_COMMAND(hwnd,cm_ParamSet,0,0,SendMessage);
		//## Test ob noch innerhalb der Messzeit
		if((CurrentTime/1000) < Topography.lMeasurementTime)
			return;
		KillTimer(GetHandle(),88);
		RestShots--;
		//## weitere Messzyklen
		if(RestShots)
			{
			sprintf(buf,"%d",RestShots);
			SetDlgItemText(GetHandle(),id_RemainderCyclus,(LPSTR)buf);
			sprintf(buf,mGetDF(),Topography.fAngleBetweenShots);
			//##Weiterfuehrung der Probe, der Timer wird durch den case Zweig
			//##cm_SteeringReady von Dlg_OnCommand (nach Ende der Steering Aktion)
			//##neu gesetzt
			Steering.Reset();
			Steering.StartUp(GetHandle(),nMotor,Device->GetId());
			Steering.StartCmdExecution(MoveToPoint,Relative,0,buf,GetHandle());
			return;
			}
		//## alle Messzyklen beendet
		sprintf(buf,"%d",RestShots);
		SetDlgItemText(GetHandle(),id_RemainderCyclus,(LPSTR)buf);
		bExeptionOccured = TRUE;
		//## Timer fuer Beepton aktivieren
		SetTimer(GetHandle(),99,2000,NULL);
		//##Zusatzzeit; Hinweisbox anzeigen
		bAdditionalTime = TRUE;
		strcpy(buf,"Me-Zeit abgelaufen ! \n\n");
		MessageBox(GetHandle(),buf,"Topographie",MBINFO);
		FORWARD_WM_COMMAND(hwnd,cm_SwitchControl,0,0,PostMessage);
		KillTimer(GetHandle(),99);
		break;
		}
	};

//## Behandlung verschiedener Nachrichten
void TTopographyExecute::Dlg_OnCommand(HWND hwnd,int id,HWND hwndCtl,UINT codeNotify)
	{
	TModalDlg*		dlg;
	static HWND   hDlgItem = NULL;
	char    			buf[2*MaxString];
	float					fExpTime;

	switch(id) {

		//##Aenderung des Messzustandes (Messung laeuft <-> keine Messung)
		case cm_SwitchControl:
		//## !!! Der Kommentar hier ist falsch an dieser Stelle !!!
		//## Er bezieht sich auf den indirekten else Zweig der Abfrage
		//## if(bControlActive), auch das Erfassen der Intensitaet wird
		//## durch Steering uebernommen und erfolgt nicht hier.
		// Starten der Regelung: Es wird zuerst die aktuelle Intensitt gemessen
		// und zum Regel-Sollwert erklrt. Dann erfolgt der Start des
		// Regelalgorithmus.

		//***** Topography was active **********************************************

		//##Ende der Messzeit wurde erreicht, Benutzer hat Hinweisfenster
		//##quittiert Timer stoppen bzw. Nachregelmechanismus anhalten
		//##Zustandsvariablen setzen, Buttonbeschriftung von stoppen auf
		//##starten aendern
		if(bControlActive)
			{
			if(Topography.bMultibleShot)
				KillTimer(GetHandle(),88);
			else
				Steering.ToggleInterrupt();
			// Herstellen des Ausgangszustandes
			bTimeFinish  = FALSE;
			bTimeRunning = FALSE;
			bAdditionalTime = FALSE;
			if(Topography.bMultibleShot)
				SetDlgItemText(GetHandle(),cm_SwitchControl,"&Topographie starten");
			else
				SetDlgItemText(GetHandle(),cm_SwitchControl,"&Regeln starten");
			StartTime = CurrentTime = 0;
			bControlActive = !bControlActive;
			SendMessage(GetHandle(),WM_COMMAND,cm_ParamSet,0);
			bControlActive = FALSE;
			//## aktiviert Button Startpositon einstellen; anstelle von !bControlActive
			//## sollte direkt TRUE angegeben werden
			EnableWindow(GetDlgItem(GetHandle(),cm_GotoWorkPoint),!bControlActive);
		break;
			}
		//***** Topography will be activated  **************************************
		//##Messung gestartet; Anzeigefelder und Buttonbeschriftungen anpassen,
		//##Zeitzaehlung beginnen, bei Mehrfachbelichtung Timer starten, bei
		//##Einfachbelichtung Steering Steuerung (fuer Nachregeln) starten
		bExeptionOccured  = FALSE;
		CurrentTime = 0;
		M_TimeString (buf,CurrentTime);
		SetDlgItemText(GetHandle(),id_RemainderTime,buf);
		StartTime = CurrentTime = GetTickCount();
		SetDlgItemText(GetHandle(),id_Text,"");
		bTimeRunning = TRUE;
		if(Topography.bMultibleShot)
		{
			// Fr Mehrfach-Belichtung
			// Start-Position wurde anfahren
			RestShots = Topography.nNumberCycle;
			SetTimer(GetHandle(),88,500,NULL);
			SetDlgItemText(GetHandle(),cm_SwitchControl,"&Topographie stoppen");
		}
		else
		{
			//##Steering auf Grundzustand setzen, Fenster, Motor und Counter
			//##an Steering uebergeben, Motorschrittweite setzen
			Steering.Reset();
			Steering.StartUp(GetHandle(),nMotor,Device->GetId());
			mSetValue(Width,Topography.fControlStep);
			StartTime = CurrentTime = GetTickCount();
			// Parameter fr Regelung laden
			//##Eintragen von Werten in die Kommandostruktur zum Nachregeln
			//##und ausfuehren des Kommandos ControlFlank durch Steering
			sprintf(buf,mGetDF(),Topography.fControlRange);
			if(Topography.bSmallAngleSide)
				Steering.StartCmdExecution(ControlFlank,SmallSide,0,buf,GetHandle());
			else
				Steering.StartCmdExecution(ControlFlank,LargeSide,0,buf,GetHandle());
			SetDlgItemText(GetHandle(),cm_SwitchControl,"&Regeln stoppen");
		}
		bControlActive = TRUE;
		EnableWindow(GetDlgItem(GetHandle(),cm_GotoWorkPoint),!bControlActive);
		break;

		//##Initalisierung
		case cm_Initialize:
		//##Counter mit Zeitintervall zum Zaehlen, maximaler Impulsrate und
		//##Messfehlerwert (0.02) initialisieren
		if(Device->IsActive())
			{
			Device->MeasureStop();
			Device->SetExposureValues(	Topography.fMaxTime,Topography.dwMaxCounts, \
																Topography.fFailure );
			Device->MeasureStart();
			}
			else
			Device->SetExposureValues(	Topography.fMaxTime,Topography.dwMaxCounts, \
																Topography.fFailure );
		//##Eintragen von Werten in die Anzeigfelder des Topographie Ausfuehren
		//##Fensters
		sprintf(buf,"%.1f",Topography.fMaxTime);
		SetDlgItemText(GetHandle(),id_ExposureTime,buf);
		RestShots = Topography.nNumberCycle;
		sprintf(buf,"%d",RestShots);
		SetDlgItemText(GetHandle(),id_RemainderCyclus,buf);
		mSetValue(Width,Topography.fControlStep);
		//##Ausgabe einer Warnmeldung, falls Motor oder Counter vor Aufruf
		//##von Topographie Ausfuehren nicht gesetzt waren
		if(!bSetupOk)
			{
			strcpy(buf,"Die Einstellungen fr Topograpie knnten\n\r");
			strcat(buf,"noch nicht exakt eingestellt sein !\n\r");
			strcat(buf," Nehmen Sie bitte Ihre Einstellungen vor.");
			}
		else strcpy(buf,"");
		SetDlgItemText(GetHandle(),id_Text,(LPSTR)buf);
		break;

		//##Button "Startposition einstellen" wurde ausgewaehlt
		case cm_GotoWorkPoint:
		// StartPosition anfahren
		//##nachfolgende Ausgabe erfolgt auch, wenn der Arbeitspunkt
		//##nicht angefahren wird, die Ausgabe sollte eigentlich
		//##nach Ueberpruefung durch die if Bedingung erfolgen
		//##alt:
		//##strcpy(buf,"Arbeitspunkt wird angefahren.");
		//##SetDlgItemText(GetHandle(),id_Text,(LPSTR)buf);
		if(bStartPointOk && Topography.bMultibleShot)
			{
			// prevents the multible relative movement
			sprintf(buf,"Die Startposition wurde bereits angefahren !");
			MessageBox(GetHandle(),buf,"Topographie",MBINFO);
			break;
			}
		//##die naechsten zwei Zeilen sind neu hier
		strcpy(buf,"Arbeitspunkt wird angefahren.");
		SetDlgItemText(GetHandle(),id_Text,(LPSTR)buf);
		//##Motorschrittweite setzen
		mSetValue(Width,Topography.fMoveStep);
		//##Button "Startposition anfahren" und
		//##"Regelung starten"/"Topographie starten" deaktivieren
		EnableWindow(GetDlgItem(GetHandle(),cm_SwitchControl),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),cm_GotoWorkPoint),FALSE);
		Steering.Reset();
		Steering.StartUp(GetHandle(),nMotor,Device->GetId());
		//##Anfahren des um dStartAngle vom aktuellen Stand entfernten Punktes
		if(Topography.bMultibleShot)
			{
			sprintf(buf,mGetDF(),Topography.dStartAngle);
			Steering.StartCmdExecution(MoveToPoint,Relative,0,buf,GetHandle());
			}
		//##Arbeitspunkt anfahren durch Makroausfuehrung
		else
			Steering.StartMacroExecution(MWorkPoint,GetHandle());
		break;

		// Arbeitspunkt wurde eingestellt
		case cm_SetupPosition:
		//##Buttons aktivieren
		EnableWindow(GetDlgItem(GetHandle(),cm_SwitchControl),TRUE);
		EnableWindow(GetDlgItem(GetHandle(),cm_GotoWorkPoint),TRUE);
		//##Ausgabe Hinweistext
		strcpy(buf,">>>>>>>>>>  X_RAY OFF  <<<<<<<<<<<<\n");
		strcat(buf,"Bitte Film-Kassette auflegen !\n");
		strcat(buf,"Die Aufnahme kann gestartet werden !");
		SetDlgItemText(GetHandle(),id_Text,(LPSTR)buf);
		bStartPointOk = TRUE;
		//##fuer die Mehrfachbelichtung setzte
		if(Topography.bMultibleShot)
			{
			//##Steering.dStartPoint wird auf die aktuelle Motorposition plus
			//##Verschiebungsweite gesetzt -> Warum??? dStartPoint wird im
			//##Topographie Zusammenhang eigentlich nicht genutzt
			Steering.dStartPoint = mGetValue(Distance) + Topography.dStartAngle;
			SendMessage(GetHandle(),WM_COMMAND,cm_ParamSet,0);
			}
		break;

		//##Steering ist mit seiner Aktion fertig
		case cm_SteeringReady:
		//##falls die Steering Aktion sich auf einen Umsetzungsschritt
		//##bei der Mehrfachbelichtung bezog, Timer fuer die naechste
		//##Runde neu initialisieren
		if(bControlActive && Topography.bMultibleShot)
			{
			StartTime = CurrentTime = GetTickCount();
			bTimeRunning = TRUE;
			SetTimer(GetHandle(),88,500,NULL);
			break;
			}
		//##Steering Aktion hat sich auf Arbeitspunkt bzw. Startposition anfahren
		//##bezogen
		FORWARD_WM_COMMAND(GetHandle(),cm_SetupPosition,0,0,PostMessage);
		break;

		//##es liegen neue Zaehlerwerte vor
		case cm_CounterSet:
		if(Topography.bMultibleShot)
			break;
		// Me-Werte speichern
		if(Topography.bMonitorUsed)
			{
			float norm;
			// Me-Signal normieren
			Topography.Monitor->GetData(norm);
			Steering.SetNorm(norm);
			}
		//##Anzeige des Zaehlerfensters aktualisieren
		Device->UpdateDisplay();
		Steering.DeviceRequest();
		// Anzeige der Regel-Gren
		FORWARD_WM_COMMAND(hwnd,cm_ParamSet,0,0,SendMessage);
		FORWARD_WM_COMMAND(GetHandle(),cm_InquireExeption,0,0,PostMessage);
		break;

		//##Nutzer hat Eingabefeld Detektor-Messzeit angewaehlt
		case id_ExposureTime:
		if(bStarted)
			hDlgItem = GetDlgItem(GetHandle(),id_ExposureTime);
		break;

		//##Nutzer hat Return gedrueckt
		case IDOK:
		//##falls das Return sich nicht auf ein Dialogfeld bezieht,
		//##Standardbehandlung durch TModalDlg
		if(!hDlgItem)
			{
			TModalDlg::Dlg_OnCommand(hwnd,id,hwndCtl,codeNotify);
			break;
			}
		// Der Nutzer hat mit Return eine neue Mezeit eingegeben
		if(hDlgItem == GetDlgItem(GetHandle(),id_ExposureTime))
			{
			//##neue Zaehlermesszeit wird eingelesen und fuer den Zaehler
			//##eingestellt, allerdings erfolgt keine Speicherung des Wertes
			//##in fMaxTime -> Wert wird zwar fuer den Detektor geaendert und
			//##auch im Topographie Fenster angezeigt, wird allerdings
			//##das Topographie Einstellen Fenster aufgerufen, so steht dort
			//##der alte Wert
			GetDlgItemText(GetHandle(),id_ExposureTime,(LPSTR)buf,10);
			fExpTime = atof(buf);
			Device->SetExposureValues( 	fExpTime,Topography.dwMaxCounts, \
																	Topography.fFailure );
			sprintf(buf,"%.1f",fExpTime);
			SetDlgItemText(GetHandle(),id_ExposureTime,(LPSTR)buf);
			//##NEU nachfolgende Zeile merkt sich den neuen Zaehlermesszeitwert
			//##Topographie global -> oben beschriebener Fehler behoben
			Topography.fMaxTime = fExpTime;
			}
		hDlgItem = NULL;
		SetFocus(GetDlgItem(GetHandle(),cm_SwitchControl));
		break;

		//##Button "Einstellungen" wurde betaetigt
		case cm_TopographyParam:
		//##Topographie Einstellungen Dialog
		dlg = (TTopographySetParam *)new TTopographySetParam(TRUE);
		dlg->ExecuteDialog(GetHandle());
		delete dlg;
		// spezifische Parameter fr einen Sensor setzen
		Device->MeasureStop();
		Device->SetExposureValues(Topography.fMaxTime,Topography.dwMaxCounts, \
		Topography.fFailure);
		Device->MeasureStart();
		//##Ausgabe der neuen Detektormesszeit
		sprintf(buf,"%.1f",Topography.fMaxTime);
		SetDlgItemText(GetHandle(),id_ExposureTime,buf);
		//##neue Motorschrittweite an den Motor weitergeben
		mSetValue(Width,Topography.fControlStep);
		break;

		//##Anzeigen im Topographie Ausfuehren Fenster aktualisieren
		case cm_ParamSet:
		sprintf(buf,"%d",RestShots);
		SetDlgItemText(GetHandle(),id_RemainderCyclus,buf);
		if(!bControlActive)
			break;
		//##abgelaufende Zeit berechnen
		if(bTimeRunning)
			{
			CurrentTime = GetTickCount();
			CurrentTime -= StartTime;
			}
		//##Drift anzeigen
		sprintf(buf,"%.2lf",mGetValue(Distance) - Steering.dStartPoint);
		SetDlgItemText(GetHandle(),id_Angle,buf);
		if(bAdditionalTime)
			{
			// Ausgabe der zustzlichen Mezeit
			M_TimeString (buf,CurrentTime);
			SetDlgItemText(GetHandle(),id_RemainderTime,buf);
			}
			else
			{
			M_TimeString (buf,CurrentTime);
			SetDlgItemText(GetHandle(),id_MeasurementTime,buf);
			}
		break;

		//##es ist moeglicherweise ein Abbruchfall eingetreten
		case cm_InquireExeption:
		if(bExeptionOccured)
			break;
		// Feststellen, ob eine Abbruch-Bedingung erfllt ist
		//##Intensitaet unter Minimumintensitaet
		if(Steering.GetIntensity() < (0.01 * Steering.fStartIntensity))
			{
			bExeptionOccured = TRUE;
			//##Beep Timer starten und Hinweisbox
			SetTimer(GetHandle(),99,2000,NULL);
			KillTimer(GetHandle(),TimerIdInformation);
			sprintf(buf,"%s","Intensitts-Abfall unter 1% !");
			MessageBox(GetHandle(),buf,"Topographie",MBINFO);
			KillTimer(GetHandle(),99);
			}
		if(Topography.bMultibleShot)
			break;
		//##es wurde um mehr als fMaxAngleEscape vom Arbeitspunkt abgewichen
		if(Topography.fMaxAngleEscape <
				fabs(Steering.dStartPoint - Steering.GetDistance()))
			{
			bExeptionOccured = TRUE;
			//##Beep Timer starten und Hinweisbox
			KillTimer(GetHandle(),TimerIdInformation);
			sprintf(buf,"%s","Regel-Bereich wurde verlassen !");
			MessageBox(GetHandle(),buf,"Topographie",MBINFO);
			KillTimer(GetHandle(),99);
			}
		//##Messzeit ist abgelaufen
		if((CurrentTime/1000) > Topography.lMeasurementTime)
			{
			bTimeFinish = TRUE;
			bExeptionOccured = TRUE;
			//##Beep Timer starten und Hinweisbox
			SetTimer(GetHandle(),99,2000,NULL);
			bAdditionalTime = TRUE;
			StartTime = GetTickCount();
			strcpy(buf,"Me-Zeit abgelaufen ! \n\n");
			strcat(buf,"Bei Besttigung wird Regelung gestoppt.");
			MessageBox(GetHandle(),buf,"Topographie",MBINFO);
			KillTimer(GetHandle(),99);
			FORWARD_WM_COMMAND(GetHandle(),cm_SwitchControl,0,0,PostMessage);
			}
		break;

		default:
		TModalDlg::Dlg_OnCommand(hwnd,id,hwndCtl,codeNotify);
		}
	};

//##Beenden des Topographie Ausfuehren Fensters
BOOL TTopographyExecute::LeaveDialog(void)
	{
	//##Wo wurde dieser Timer gesetzt? Ist er fuer die Topographie ueberhaupt
	//##relevant?
	KillTimer(GetHandle(),TimerIdInformation);
	//##"BeepTimer" stoppen
	KillTimer(GetHandle(),99);
	//##aktuelle Motor zuruecksetzen
	mPopSettings(ThisPosition);
	//##Detektor zuruecksetzen
	Device->MeasureStop();
	Device->PopSettings();
	Device->SetHost(NULL);
	Device->MeasureStart();
	if(!bSetupOk)
		Topography.Sensor = NULL;
	return TRUE;
	};

//******************************************************************************
//##Konstruktor; restrictions gibt an, ob das Fenster von Topographie Ausfuehren
//##aus aufgerufen wurde
TTopographySetParam::TTopographySetParam(int restrictions) : TModalDlg("TopographyParam")
	{
	nRestrictions = restrictions;
	//##falls Detektor noch nicht gesetzt, auf aktuellen Detektor setzen
	if(Topography.Sensor == NULL)
		{
		Topography.Sensor 			= lpDList->DP();
		Topography.Monitor 			= Topography.Sensor;
		Topography.bMonitorUsed = FALSE;
		}
	//##falls noch kein Motor gesetzt, wenn moeglich auf Omega setzen, sonst auf
	//##den aktuellen
	if(Topography.nMotor < 0)
		if(mlIsAxisValid(Omega))
			Topography.nMotor = mlGetIdByName(Omega);
		else
			Topography.nMotor = mlGetAxis();
	};

#pragma argsused
//##Initalisierung
BOOL TTopographySetParam::Dlg_OnInit(HWND hwnd,HWND hwndCtl,LPARAM lParam)
	{
	int		i;
	if(nRestrictions)
		{
		//##Wird das Einstellungsfenster von dem Topographie Fenster aus
		//##aufgerufen, so ist nur fuer einige Werte die Eingabe moeglich. Alle
		//##anderen Eingabefelder werden hier deaktiviert und erscheinen dann
		//##im Fenster ausgegraut.
		EnableWindow(GetDlgItem(GetHandle(),id_ChooseMotor),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),cm_RotateMotor),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),id_ChooseDevice),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),id_WorkPoint),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),id_Shots),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),id_MaxAngleEscape),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),id_StepWidth),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),id_From),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),id_MultibleShot),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),id_WorkPoint),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),id_ControlRange),FALSE);
		EnableWindow(GetDlgItem(GetHandle(),id_MoveStep),FALSE);
		// EnableWindow(GetDlgItem(GetHandle(),id_ControlStep),FALSE);
		}
	nMotor  = Topography.nMotor;
	Device  = Topography.Sensor;
	Monitor = Topography.Monitor;
	//##Motoren- /Detektorenauswahlliste aufbauen
	hDeviceList = GetDlgItem(GetHandle(),id_ChooseDevice);
	hMotorList = GetDlgItem(GetHandle(),id_ChooseMotor);
	i = 0;
	while(i <= lpDList->LastId())
		ComboBox_AddString(hDeviceList,lpDList->DP(i++)->Characteristic);
	i = -1;
	while(++i < mlGetAxisNumber())
		{
		mlSetAxis(i);
		ComboBox_AddString(hMotorList,mGetAxisName());
		}
	mlSetAxis(nMotor);
	FORWARD_WM_COMMAND(hwnd,cm_ParamSet,hwndCtl,0,SendMessage);
  return TRUE;
	};

//##Bearbeitung verschiedener Nachrichten an das Topographie Einstellen Fenster
void TTopographySetParam::Dlg_OnCommand(HWND hwnd,int id,HWND hwndCtl,UINT codeNotify)
	{
	char    buf[MaxString];
	float   valuef;
	int     idx;

	switch(id)
		{
		//##aktualisieren der Anzeige/Eingabefelder
		case cm_ParamSet:
		ComboBox_SelectString(hMotorList,0,mGetAxisName());
		ComboBox_SelectString(hDeviceList,0,Device->Characteristic);
		SetDlgItemText(GetHandle(),id_Unit,mGetAxisUnit());
		//##SetFocus funktioniert nicht, "verbiegt" aber handel,
		//##so dass getHandle in Folge fehlerhaften Wert liefert
		//##
		//##SetFocus(GetDlgItem(GetHandle(),IDCANCEL));
		valuef = Topography.fWorkPoint*100;
		if(Topography.bSmallAngleSide) valuef *= -1.0;
		sprintf(buf,"%.1f",valuef);
		SetDlgItemText(GetHandle(),id_WorkPoint,buf);
		SetDlgItemInt(GetHandle(),id_Shots,Topography.nNumberCycle,FALSE);
		sprintf(buf,"%.1f",Topography.fMaxAngleEscape);
		SetDlgItemText(GetHandle(),id_MaxAngleEscape,buf);
		//## Schrittweite beim Regeln
		sprintf(buf,mGetSF(),Topography.fControlStep);
		SetDlgItemText(GetHandle(),id_ControlStep,buf);
		sprintf(buf,mGetSF(),Topography.fMoveStep);
		SetDlgItemText(GetHandle(),id_MoveStep,buf);
		sprintf(buf,"%.1f",Topography.fControlRange*100);
		SetDlgItemText(GetHandle(),id_ControlRange,buf);
		sprintf(buf,"%.1f",Topography.fMaxTime);
		SetDlgItemText(GetHandle(),id_MaxTime,buf);
		//##da lMeasurmentTime ein long Wert ist, darf hier nicht nach int
		//##gecastet werden, anstelle der Formatierungsangabe %.d fuer int
		//##muss %.ld fuer long int stehen
		//## alt: sprintf(buf,"%.d",(int)(Topography.lMeasurementTime/60l));
		//##NEU:
		sprintf(buf,"%.ld",(Topography.lMeasurementTime/60l));
		SetDlgItemText(GetHandle(),id_MeasurementTime,buf);
		sprintf(buf,"%lu",Topography.dwMaxCounts);
		SetDlgItemText(GetHandle(),id_MaxCounts,buf);
		CheckDlgButton(GetHandle(),id_MultibleShot,Topography.bMultibleShot);
		sprintf(buf,"%.1f",Topography.fAngleBetweenShots);
		SetDlgItemText(GetHandle(),id_StepWidth,buf);
		sprintf(buf,"%.1f",Topography.dStartAngle);
		SetDlgItemText(GetHandle(),id_From,buf);
		//##Falls Fenster von Topographie Ausfuehren aufgerufen wird
		//## nRestriction == true, vom Hauptmenue aus nRestriction == false
		//##-> Einfluss auf Aktivierung von Eingabefeldern
		if(!nRestrictions)
			{
			EnableWindow(GetDlgItem(GetHandle(),id_Shots),Topography.bMultibleShot);
			EnableWindow(GetDlgItem(GetHandle(),id_From),Topography.bMultibleShot);
			EnableWindow(GetDlgItem(GetHandle(),id_StepWidth),Topography.bMultibleShot);
			EnableWindow(GetDlgItem(GetHandle(),id_WorkPoint),!Topography.bMultibleShot);
			EnableWindow(GetDlgItem(GetHandle(),id_ControlRange),!Topography.bMultibleShot);
			}
		break;

		//##Motor Auswahlbox wurde angewaehlt
		case id_ChooseMotor:
		switch(codeNotify)
			{
			//##es wurde etwas ausgewaehlt
			case CBN_SELCHANGE:
			idx = ComboBox_GetCurSel(hMotorList);
			//##ueberprueft ob tatsaechlich ein neuer Motor gewaehlt wurde
			if((nMotor != idx) && (idx != CB_ERR))
				{
				//##initialisiert den ausgewaehlten Motor, aktualisiert die Anzeige
				mlSetAxis(idx);
				nMotor = mlGetAxis();
				FORWARD_WM_COMMAND(hwnd,cm_ParamSet,0,0,SendMessage);
				}
			}
		break;

		//##Detektor Auswahlbox wurde angewaehlt
		case id_ChooseDevice:
		switch(codeNotify)
			{
			//##es wurde etwas eingegeben/ausgewaehlt
			case CBN_SELCHANGE:
			idx = ComboBox_GetCurSel(hDeviceList);
			if((Device != lpDList->DP(idx)) && (idx != CB_ERR))
				{
				//##setzt den ausgewaehlten Detektor als aktuellen Detektor ein,
				//##aktualisiert die Anzeige
				Device = lpDList->DP(idx);
				FORWARD_WM_COMMAND(hwnd,cm_ParamSet,0,0,SendMessage);
				}
			}
		break;

		//##Zu Zeit nicht genutzt (ehemals gab es neben der Motorauswahl-Box
		//##Schalter, mit dem sich die Auswahl rotieren lassen sollte)
		case cm_RotateMotor:
		mlSetAxis(nMotor+1);
		nMotor = mlGetAxis();
		FORWARD_WM_COMMAND(hwnd,cm_ParamSet,0,0,SendMessage);
		break;

		//##Aktivieren bzw. Deaktivieren der Mehrfachbelichtungs Checkbox
		//##Bei Aktivierung:
		//##Eingabefelder im Bereich Mehrfachbelichtung werden aktiviert
		//##Arbeitspunkt Lage und Belichtungsregelung Bereich deaktiviert
		//##Bei Deaktivierung entsprechend umgekehrte Aktivierung/Deaktivierung
		//##der Eingabefelder
		case id_MultibleShot:
		Topography.bMultibleShot = IsDlgButtonChecked(GetHandle(),id_MultibleShot);
		EnableWindow(GetDlgItem(GetHandle(),id_Shots),Topography.bMultibleShot);
		EnableWindow(GetDlgItem(GetHandle(),id_From),Topography.bMultibleShot);
		EnableWindow(GetDlgItem(GetHandle(),id_StepWidth),Topography.bMultibleShot);
		EnableWindow(GetDlgItem(GetHandle(),id_WorkPoint),!Topography.bMultibleShot);
		EnableWindow(GetDlgItem(GetHandle(),id_ControlRange),!Topography.bMultibleShot);
		break;

		default:
		TModalDlg::Dlg_OnCommand(hwnd,id,hwndCtl,codeNotify);
		}
	};

//##Testet nacheinander fuer alle Eingabefelder, ob die Werte im
//##Wertebereich liegen (im Moment nur Test auf Ungleichheit mit Null,
//##siehe auch Fehlerprotokoll) und uebernimmt sie dabei als Variablenwerte
//##von TTopographie
//##Einzelfaelle ausfuehren
BOOL TTopographySetParam::CanClose(void)
	{
	char    buf[MaxString];
	float   valueF;
   //##wird nicht mehr benoetigt siehe Aenderung zur Uebernahme von nNumberCycle
	//##short   valueI;
	long    valueL;

	//##Feststellen ob Mehrfachbelichtung ausgewaehlt wurde
	Topography.bMultibleShot = IsDlgButtonChecked(GetHandle(),id_MultibleShot);

	//##Regelbereich nach Intensitaet in Prozent
	GetDlgItemText(GetHandle(),id_ControlRange,buf,10);
	valueF = atof(buf);
	//##ein negativer Prozentwert macht keinen Sinn, die Werte muessen >= 0.01
	//##sonst gibt es bei erneuter Darstellung Probleme, da maximal vier Zeichen
	//##zur Verfuegung stehen
	//## alt: if(0.0 != valueF)
	//##NEU:
	if(valueF >= 0.01)
		{
		Topography.fControlRange = valueF/100.0;
		}
		else return FALSE;

	//##Abstand bei Mehrfachbelichtungen
	GetDlgItemText(GetHandle(),id_StepWidth,buf,10);
	valueF = atof(buf);
	//##hier gibt es Probleme bei der erneuten Darstellung, falls der Betrag des
	//##eingebenen Wertes < 0.001
	//## alt: if(0.0 != valueF)
	//##NEU:
	if((valueF >= 0.001) || (valueF <= -0.001))
		{
		Topography.fAngleBetweenShots = valueF;
		}
		else return FALSE;

	//##fuer die Mehrfachbelichtung die relative Startposition zur aktuellen
	//##Motorposition
	GetDlgItemText(GetHandle(),id_From,buf,10);
	Topography.dStartAngle = atof(buf);

	//##Regel Schrittweite
	GetDlgItemText(GetHandle(),id_ControlStep,buf,10);
	valueF = atof(buf);
	//##negative Werte sind hier nicht sinnvoll, bei der erneuten Darstellung
	//##gibt es Probleme, falls der Betrag des eingegebenen Wertes < 0.00001
	//## alt: if(0.0 != valueF)
	//##NEU:
	if( valueF >= 0.00001 )
		{
		Topography.fControlStep = valueF;
		}
		else return FALSE;

	//##Schrittweite zum Anfahren des Arbeitspunktes
	GetDlgItemText(GetHandle(),id_MoveStep,buf,10);
	valueF = atof(buf);
	//##negative Werte sind hier nicht sinnvoll, bei der erneuten Darstellung
	//##gibt es Probleme, falls der Betrag des eingegebenen Wertes < 0.0001
	//## alt: if(0.0 != valueF)
	//##NEU:
	if( valueF >= 0.0001 )
		{
		Topography.fMoveStep = valueF;
		}
	else return FALSE;

	//##maximale Abweichung vom Arbeitspunkt
	GetDlgItemText(GetHandle(),id_MaxAngleEscape,buf,10);
	valueF = atof(buf);
	//##negative Werte sind hierfuer nicht sinnvoll, ist der eingegebene
	//##Wert < 0.01, so wird er bei der naechsten Darstellung als 0.0 angezeigt
	//## alt:if(0.0 != valueF)
	//##NEU:
	if (valueF >= 0.01 )
		{
		Topography.fMaxAngleEscape = valueF;
		}
	else return FALSE;

	//##Belichtungszeit
	GetDlgItemText(GetHandle(),id_MeasurementTime,buf,10);
	valueL = atol(buf);
	//##eine negative Belichtungszeit macht keinen Sinn, da es sich
	//##um einen long Wert handelt, sollte nicht mit einem double Wert
	//##(0.0) verglichen werden
	//## alt: if(0.0 != valueL)
	//##NEU:
	if ( (valueL > 0) )
		{
		Topography.lMeasurementTime = valueL * 60l;
		}
	else return FALSE;

	//##Anzahl Zyklen fuer Mehrfachbelichtung
	//##eigentlich wird dieser Wert nur benoetigt, falls Mehrfachbelichtung
	//##angekreuzt ist
	GetDlgItemText(GetHandle(),id_Shots,buf,10);
	//##nNumberCycle ist vom Type WORD (vorzeichenlos, max. 65535)
	//##im Eingabefeld koennen aber auch groessere Werte und negative Werte
	//##eingegeben werden, beides fuehrt zu Ueberlaeufen
	//##alt: valueI = (short)atoi(buf);
	//##alt: if(0.0 != valueI)
	//##NEU:
	valueL = atol(buf);
	if( (valueL > 0) && (valueL <= 65535) )
		{
		Topography.nNumberCycle = valueL;
		}
	else return FALSE;

	//##Lage des Arbeitspunktes relativ zur aktuellen Intensitaet
	GetDlgItemText(GetHandle(),id_WorkPoint,buf,10);
	valueF = atof(buf);
	//##ist der Betrag < 0.001 gibt es bei erneuter Darstellung Probleme
	//## alt: if(0.0 != valueF)
	//##NEU:
	if( (valueF >= 0.001) || (valueF <= -0.001))
		{
		Topography.bSmallAngleSide = (valueF < 0.0);
		Topography.fWorkPoint = fabs(valueF/100);
		}
	else return FALSE;

	//##maximale Zaehlzeit des Detektors
	GetDlgItemText(GetHandle(),id_MaxTime,buf,10);
	valueF = atof(buf);
	//##negative Werte sind nicht sinnvoll, ist der Wert < 0.0001 ist
	//##die Neudarstellung fehlerhaft
	//## alt: if(0.0 != valueF)
	//##NEU:
	if ( valueF >= 0.0001 )
		{
		Topography.fMaxTime = valueF;
		}
	else return FALSE;

	//##maximale Zaehlercounts
	GetDlgItemText(GetHandle(),id_MaxCounts,buf,10);
	valueL = atol(buf);
	//##ein negativer Wert ist hier nicht sinnvoll, da es sich um einen
	//##long Wert handelt, sollte nicht mit einem double Wert (0.0)
	//##verglichen werden
	// alt: if(0.0 != valueL)
	//##NEU:
	if( valueL > 0 )
		{
		Topography.dwMaxCounts = valueL;
		}
	else return FALSE;

	Topography.nMotor	  = nMotor;
	Topography.Sensor   = Device;
	Topography.Monitor  = Monitor;
	return TRUE;
	};

//##Aufraeumen beim Schliessen des Eistellungen Topographie Fenster
BOOL TTopographySetParam::LeaveDialog(void)
	{
	mlSetAxis(nOldMotor);
	return TRUE;
	};
//******************************************************************************
// __LastLine__
