// (C) Copyright 1993,1994 by Heiko Damerow MPG AG "Roentgenbeugung"
// Datei m_scan.h

//! kommentiert von diffr.gruppe

#ifndef __M_XSCAN_H
#define __M_XSCAN_H

#define CLIENTWINDOWSTYLE WS_CHILD|WS_CLIPCHILDREN|WS_VSCROLL|WS_HSCROLL

#include "m_data.h"
#include "m_psd.h"



//! Speichern, ohne nach Dateinamen zu fragen
const int NoAskForSave = 0x9900;

//! Speichern mit Abfrage des Dateinamens
const int SaveWithNewName = 0x9901;

//! Anzahl der max. unterschiedlichen Schrittweiten (dyn. Schrittweite)
const int nMaxScaleIdx = 10;

class TScanParameters
{
    //! einlesen der Ini-daten zu [Scan] und setzen von default-werten fuer
    //! StepScan
public:
    TScanParameters( void );

    //! beinhaltet Nummer der aktuellen Achse
    int nMotor;

    //! Speicherformat fuer Kurven
    TSaveFormat eSaveFormat;

    //! Art des aktuellen Scan(StandardScan, Omegea2Theta o. ContinuousScan)
    TxScanType eScanType;

    //! Speichern nach Beendigung der Messung
    BOOL bSaveOnReady;

    //! Anzahl der Messpunkte(nur bei Cont.Scan)
    int nCollectionAmount;

    //! Startwinkel des Scans
    double dArgumentMin,
    //! Endwinkel des Scans
    dArgumentMax,
    //! Schrittweite waehrend des Scans
    dArgumentWidth;

    //! Geschwindigkeit des aktuellen Motors
    double dSpeed;

    //! aktueller Detektor zum Messen
    LPDevice Device;

    int nMFunctionId; // Typ der Mess-Funktion
    WORD nNumberCycle; // Anzahl der Mess-Zyklen
    float fFailure; // Zur Konfiguration des Zaehlers
    float fMaxTime; // Maximale Messzeit
    DWORD dwMaxCounts; // Maximale Anzahl Impulse

    //! Detektor fuer Monitor
    LPDevice Monitor;

    //! Speichern waehrend der Messung
    BOOL bSaveContinuous;

    //! Monitor-detektor benutzen
    BOOL bMonitorUsed;

    //! gibt an ob dynamische Schrittweitensteuerung aktiv
    BOOL bDynamicStepWidth;

    //! Anzahl der verschied. Schrittweiten
    int nStepScaleLevels;

    //! Liste enthaelt in den ersten 10 werten die intensitaeten und in den
    //! zweiten 10 werten den zughoer. Schrittweitenfaktor
    float Scaler[ 20 ];
};

class TScan : public TScanParameters, public TPlotData
{
    friend class TSetScanParameters;
    friend class TSteering;
    friend class TScanCmd;
    friend class TSetupScanCmd;
    friend class TSaveDataCmd;

public:
    TScan( void );
    ~TScan();

protected:
    //! laden einer gespeicherten Kurve (.crv/.bk/.dtn)
    //! ?? input.par. nicht impl.
    //! Rueckkehrcode false wenn Fehler beim oeffnen der Datei, fehlerhaften
    //! Headerinfo. oder keine Eintraege nach[DATA] sonst true
    BOOL LoadOldData( int );

    //! speichern einer .crv Datei
    //! ask== NoAskForSave(0x9900) => speichert unter Namen aus FileName
    //! ohne Abfrage
    //! == SaveWithNewName(0x9901) => speichert unter Namen, der in der
    //! Dialogbox erst noch enzugeben ist
    //! Rueckkehrcode False wenn Fehler bei anlegen der Datei oder bei
    //! speichern der Headerinfo. sonst true
    BOOL SaveFile( int );

    //! haengt aktuelles tripel der Messkurve an Datei mit FileName an
    //! aufruf nur bei kontinuierlichem Speichern
    //! Rueckkehrcode False wenn fehler bei oeffnen der Datei sonst true
    BOOL UpdateFile();

    //! ?? scheinbar ueberfluessig
    //! rueckkehrcode: immer true
    BOOL SaveDataBase( void );

    //! darstellen des StepScanfenster
    void Create( HWND hwnd );

    //! erhaelt Messpunkt, stellt ihn dar, speichert ihn ggf.
    void CounterSetRequest( LPARAM );

    //! behandelt Ende des Messvorgangs
    void SteeringReady( LPARAM );

    //! passt x+y-Scale des Koord.sys an neue Werte an
    void SetRanges( void );

    //! ?? scheinbar ueberfluessig
    void DrawMenu( void );

    //! loescht aktuelle Kurve + baut StepScanfenster neu auf
    //! rueckkehrcode: immer true
    BOOL New( void );

    //! laden der Messungsdaten aus einer .crv bzw. .bk datei
    //! wird von TScan::LoadOldData() in m_scan.cpp aufgerufen
    //! hFile ist Filehandle der Messdatei
    //! Rueckkehrcode False wenn Datei zu kurz oder [HEADER] nicht vorhanden
    //! sonst true
    BOOL LoadMeasurementInfo( HFILE );

    //! Speichern bzw. Aktualisieren der Headerinformation
    //! Par.==0 => neu erstellen
    //! ==1 => aktualisieren
    //! Rueckkehrcode False wenn Fehler bei speichern sonst true
    BOOL SaveMeasurementInfo( BOOL );

    //! aufruf des Stepscanvorgangs
    void InitializeTask( WPARAM, LPARAM );

    //! behandelt einige Menuekommandos
    void InitializeDlg( WPARAM, LPARAM );

    //! unterbricht Scanvorgang
    void Interrupt();

    //! ?? scheinbar ueberfluessig
    void StopMeasurement();

private:
    //! ?? scheinbar ueberfluessig
    BOOL bMacroRunning;

    //! setzt Titel im Stepscanfenster
    //! rueckkehrcode: immer true
    BOOL SetTitle( void );

    //! ?? scheinbar ueberfluessig
    BOOL NewFile();

    //! ?? scheinbar ueberfluessig
    void ExecuteNextCmd( void );

    //! Test ob Stepscanfenster schliessbar
    //! Rueckkehrcode False wenn kein Fenster geoeffnet war, sonst immer True
    BOOL CanClose( void );

    //! setzt Untermenue an Mausposition
    void rButtonDown( LPARAM );

    //! liefert Klassennamen
    LPCSTR ClassName()
    {
        return "Scan";
    };

    //! liefert Charakteristik, ?? scheinbar ueberfluessig
    LPCSTR GetCharacteristic()
    {
        return "Curve";
    };

    //! ?? scheinbar ueberfluessig
    double GetArgumentMin( void )
    {
        return dArgumentMin;
    };

    //! ?? scheinbar ueberfluessig
    double GetArgumentMax( void )
    {
        return dArgumentMax;
    };

    //! soll Scanstart vor Aufruf einer Einstellungsbox verhindern
    //! (Fehler 2)
    BOOL bScanSetupOk;

};

class TAreaScanCCDParameters
{

  public:

         TAreaScanCCDParameters( void );

         //! enthaelt Nummer des Omegamotors in der Motorenliste
         int nOmega;

         //! enthaelt Anfangs- und Endwinkel sowie die Schrittweite des Omegamotors
         //! fuer einen Areascan
         double dOmegaMin, dOmegaMax, dOmegaWidth;

         //! gibt an in welcher Form die Messwerte in Datei abgelegt werden sollen
         //! (nur StandardFile verwendet)
         TSaveFormat eSaveFormat;

         //! Speichern nach Beendigung der Messung
         BOOL bSaveOnReady;

         //! gibt an ob ein CCD angeschlossen
         BOOL bCCDDevice;

         //! gibt ausgewaehlten Detektor an
         LPDevice Device;

         //! enthaelt die maximale Messzeit
         float fMaxTime;

         //! enthaelt die Totzeit
         float fDelay;

         //! enthlt Hardwaremodus
         int iHardwareMode;

         //! setzt den Detektor, dessen id-nummer dem Param. id entspricht
         //! dabei wird bPsdDevice gesetzt, wenn der Detektor ein Psd ist
         void SetDevice( int );

};

class TAreaScanCCD : public TAreaScanCCDParameters, public TPlotData //, public TMDIWindow

{

  public:

         TAreaScanCCD( void );
         ~TAreaScanCCD( void );

  protected:

         //! loescht aktuelle datenbasis + baut AreaScanfenster neu auf
         //! Rueckkehrcode immer True
         BOOL New( void );

         //! Darstellen des AreaScanCCD-Fenster
         void Create( HWND );

         //! aufruf wenn rechte Maustaste gedrueckt
         //! Fenstermenue an aktueller Mausposition ausgeben
         //! eingabeparam. aktuelle Mauspos.low(8) x-pos
         //!                                high(8) y-pos.
         void rButtonDown( LPARAM );

         //! behandelt einige Menuekommandos
         void InitializeDlg( WPARAM, LPARAM );

         //! wird aufgerufen aus m_main.cpp bei cm_InitializeScan
         //! startet den Scanvorgang
         void InitializeTask( WPARAM, LPARAM );

  private:

         //! gibt Nummer der Kurve in Datenbasis an, die gerade aktiv ist
         int nActiveScan;

         //! setzt Titel des AreaScanCCD-Fensters
         //! Rueckkehrcode immer True
         BOOL SetTitle( void );

};

class TSetupAreaScanCCD : public TModalDlg
{

  public:

         TSetupAreaScanCCD( TAreaScanCCD* );

  private:

         //! gibt ausgewaehlten Detektor an
         LPDevice Device;

         //! Pointer auf AreaScanCCD-Fenster
         TAreaScanCCD* Scan;

         //! Handle der Detektorauswahlbox
         HWND hDeviceList;

         //! realisiert test der Dialogboxinhalte auf ihre konsistenz vor verlassen
         //! der Dialogbox (durch OK-Button)
         //! Rueckkehrcode false wenn die Dialogbox nicht geschlossen werden kann
         //! (Werte inkonsistent) sonst true
         BOOL CanClose( void );

         //! behandelt Kommandos in der Dialogbox
         void Dlg_OnCommand( HWND, int, HWND, UINT );

         //! setzt Defaultwerte der Dialogbox bei Aufruf der Box
         //! Rueckkehrcode immer true
         BOOL Dlg_OnInit( HWND, HWND, LPARAM );

         //! wird bei verlassen der Dlg.box aufgerufen (in dieser Dlg.box keine
         //! Bedeutung) da auch geerbt
         //BOOL LeaveDialog( void );
};

class TAreaScanParameters
{

public:
    TAreaScanParameters( void );

    //! enthaelt Nummer des Omega- sowie Thetamotors in der Motorenliste
    int nOmega, nTheta;

    //! enthaelt Anfangs- und Endwinkel sowie die Schrittweite des Omegamotors
    //! fuer einen Areascan
    double dOmegaMin, dOmegaMax, dOmegaWidth;

    //! enthaelt Anfangs- und Endwinkel , die Schrittweite sowie den Offset des
    //! Thetamotors fuer einen Areascan
    //! NEU dThetaStep
    double dThetaMin, dThetaMax, dThetaWidth, dThetaOffset;

    //! gibt das Schrittverhaeltnis zwischen Omega und Theta an
    double dMoveRelation,
    //! gibt den Thetabereich eines Spektrums an
    dThetaWindow;

    //! gibt den Scantyp an (omega2Theta/StandardScan)
    TxScanType eScanType;

    //! gibt an in welcher Form die Messwerte in Datei abgelegt werden sollen
    //! (nur StandardFile verwendet, erweitert um bkFile, dtnFile)
    TSaveFormat eSaveFormat;

    //! Speichern nach Beendigung der Messung
    BOOL bSaveOnReady;

    //! Speichern waehrend der Messung
    BOOL bSaveContinuous;

    //! gibt an ob ein Psd angeschlossen
    BOOL bPsdDevice;

    //! bisher ueberfluessig (bei anderen Speicherformaten verwendet)
    BOOL bThetaAddsPsd;

    //! gibt ausgewaehlten Detektor an
    LPDevice Device;

    //! zeigt an ob AreaScan mit Absorber durchzufuehren
    BOOL bAbsorberUsed;

    //! gibt an wieviele Kanaele des Psd zu einem zusammenzufassen sind
    int nAddedChannels;

    float fFailure; // Zur Konfiguration des Zaehlers
    float fMaxTime; // Maximale Messzeit
    DWORD dwMaxCounts; // Maximale Anzahl Impulse

    //! gibt ausgewaehlten Monitordetektor an (ist bisher nicht auswaehlbar)
    LPDevice Monitor;

    //! zeigt an ob AreaScan mit Monitor durchzufuehren (ueberfluessig siehe
    //! Monitor)
    BOOL bMonitorUsed;

    //! gibt an welche zusatzdaten zu einem Spektrum im Scanreport zu speichern
    //! sind , zur zeit nur IntegralIntensity moeglich
    TNotifyValue eNotifyValue;

    //! setzt den Detektor, dessen id-nummer dem Param. id entspricht
    //! dabei wird bPsdDevice gesetzt, wenn der Detektor ein Psd ist
    void SetDevice( int );
};

class TAreaScan : public TAreaScanParameters, public TPlotData
{
    friend class TSteering;
    friend class TSetupScanCmd;
    friend class TScanCmd;
    friend class TSetupAreaScanCmd;
    friend class TAreaScanCmd;
    friend class TSaveDataCmd;
    friend class TChooseScan;
    friend class TAquisition;

public:
    TAreaScan( void );
    ~TAreaScan( void );

protected:

    //! Funktion wird bei WM_CREATE aus WndProc aufgerufen
    //! testet ob weniger als 5 Fenster geoeffnet sind liefert dann ein true
    //! als Rueckkehrcode ansonsten ein false
    //! wahrscheinlich ueberfluessig (wird geerbt)
    BOOL CanOpen( void );

    //! test ob Fenster geschlossen werden kann
    //! -test ob Messung aktiv, wenn ja, dann zuerst Messung unterbrechen
    //! (Interrupt())
    //! -test ob geaenderte Daten, wenn ja, dann Speicher-Anfrage
    //! Rueckkehrcode False wenn kein Fenster goeffnet war sonst immer True
    //! wahrscheinlich ueberfluessig (wird geerbt)
    BOOL CanClose( void );

    //! liest gespeicherte Kurve (.psd)
    //! ?? inputparam nicht implem.
    //! Rueckkehrcode false wenn Fehler beim oeffnen der Datei, Extension ungl.
    //! .psd, fehlerhaften Headerinfo. oder keine Eintraege nach[DATA]
    //! sonst auch false (fehler)
    BOOL LoadOldData( int );

    //! Darstellen des AreaScan-Fenster
    void Create( HWND );

    //! wahrscheinlich ueberfluessig da vererbt von TMDIWindow
    void DrawMenu( void );

    //! Funktion behandelt Fenstermenuepunkte
    LRESULT CallLocalAction( WPARAM, LPARAM );

    //! aufruf wenn rechte Maustaste gedrueckt
    //! Fenstermenue an aktueller Mausposition ausgeben
    //! eingabeparam. aktuelle Mauspos.low(8) x-pos
    //! high(8) y-pos.
    void rButtonDown( LPARAM );

    //! aufruf wenn linke Maustaste losgelassen wurde(d.h. Endpunkt erreicht)
    //! wird in 2 Faellen aktiv
    //! - messen von entfernung zwischen zwei Punkten ueber TPlotData
    //! - bei Dataaquisition(nMouseAction==100) zusaetzlich Kommando senden
    void lButtonUp( WPARAM, LPARAM );

    //! aufruf wenn linke Maustaste gedrueckt wurde
    //! (d.h. Startpunkt wurde gewaehlt)
    //! wird in 2 Faellen aktiv
    //! - messen von entfernung zwischen zwei Punkten (nMouseAction=0)
    //! - bei Dataaquisition (nMouseAction=100)
    void lButtonDown( WPARAM, LPARAM );

    //! wird aufgerufen aus m_main.cpp bei cm_InitializeScan
    //! startet den Scanvorgang
    void InitializeTask( WPARAM, LPARAM );

    //! behandelt die Kommandos 'Setup Measurement...' und 'Inquire Data'
    //! des Kindfenster-menue ; aufruf aus m_main.cpp DoCommandsChild()
    void InitializeDlg( WPARAM, LPARAM );

    //! wahrscheinlich ueberfluessig
    void StopMeasurement( void );

    //! Fkt. wird bei unterbrechen/fortsetzen eines AreaScans aufgerufen oder
    //! zum ausschalten einer kont. Anzeige
    void Interrupt( void );

    //! ??
    //! Aufruf aus CallLocalAction bei ExternalSync
    //! Dialogbox unvollstaendig + gesamte Funktionalitaet ExternalSync nur in
    //! Ansaetzen implementiert
    void ExternSychronized( void );

    //! aufruf aus CallLocalAction zur PsdCalibrierung
    void CalibratePsd( void );

    //! bildn. Darstellung ausschl. in TPlotData verwendet
    BOOL SetMeasurementArea( TKSystem & );

    //! loescht aktuelle datenbasis + baut AreaScanfenster neu auf
    //! Rueckkehrcode immer True
    BOOL New( void );

    //! Funktion speichert einen Messdatensatz
    //! 1.Parameter gibt an ob neuer Dateiname zu waehlen ist (SaveWithNewName)
    //! oder ob automatisch unter altem Namen (NoAskForSave) gespeichert werden
    //! soll
    //! Rueckkehrcode False wenn Fehler bei anlegen der Datei oder bei
    //! speichern der headerinfos sonst immer true
    BOOL SaveFile( int );

    //! speichert die zur Datenbasis gehoerige Scanreportkurve in einer Datei
    //! unter dem aktuellen Dateinamen mit der Extension .rep ab
    //! Rueckkehrcode immer true
    BOOL SaveReport( void );

    //! Funktion laedt die zu einer .psd datei gehoerige Scanreportkurve
    //! Rueckkehrcode False wenn Fehler bei oeffnen der Datei oder wenn weniger
    //! als 3 Punkte in der Datei waren
    //! true sonst
    //! wird nicht benutzt
    BOOL LoadReport( void );

    //! zeichnet in abhaengigkeit des eingestellten Darstellungstyps das
    //! AreaScanfenster neu
    void UpdateWnd( int );

    //! haengt aktuelles Kurve an Datei mit FileName an
    //! aufruf nur bei kontinuierlichem Speichern
    //! Rueckkehrcode immer true
    BOOL UpdateFile( void );

    //! setzt die Achsen-min. und -max. abhaengig von der Darstellungsart
    void SetRanges( void );

    //! Funktion liefert den absoluten Thetawert als Rueckkehrcode
    //! 1.Param. scid gibt Kurvennr. an, deren absoluter Thetawert gesucht ist
    //! nur bei RL-Bitmap benutzt
    double GetThetaOffset( int );

    //! liefert den Klassennamen zurueck
    LPCSTR ClassName()
    {
        return "AreaScan";
    };

    //! liefert Klassencharakteristik (nicht benutzt)
    LPCSTR GetCharacteristic()
    {
        return "AreaScan";
    };

    //! behandelt eigentlichen Scanablauf sowie die kont. Anzeige ,Calibrate...
    //! aufruf aus m_main.cpp wenn cm_CounterSet
    void CounterSetRequest( LPARAM );

    //! diese Fkt. schaltet die kont. Spektrumsanzeige fuer Psd an oder aus
    //! 1.Param. type== PsdEnergyData(3000) => kont. EnergieSpektrum an/aus
    //! PsdPositionData(3001) => kont. ImpulsSpekrum an/aus
    //! (99) => schaltet kont. Spektrum aus
    //! (0) => schaltet kont.Spektrum aus und dieses spaeter
    //! wieder ein
    //! nicht implementiert aber benutzt
    void ShowSensorContinuous( BOOL );

    //! wird augerufen nachdem AreaScan abgeschlossen wurde
    //! aus m_main.cpp bei cm_SteeringReady
    void SteeringReady( LPARAM );

    //! scheinbar ueberfluessig
    double GetArgumentMin( void )
    {
        return dOmegaMin;
    };
    double GetArgumentMax( void )
    {
        return dOmegaMax;
    };

    //! nur bei benutzung von 0 dim. Detekt. von Bedeutung
    //! zeigt an ob der Detektor den Thetabereich durchlaufen hat
    BOOL bThetaScanCompleted;

private:

    //! gibt Nummer der Kurve in Datenbasis an, die gerade aktiv ist
    int nActiveScan;

    //! scheinbar ueberfluessig
    HWND hOldControlWnd;

    //! zeigt an, ob kont. Spektrumsanzeige aktiv ist
    BOOL bShowPsdContinuous;

    //! zeigt an, ob die Psdkalibrierung gerade durchgefuehrt wird
    BOOL bCalibrationActive;

    //! scheinbar ueberfluessig
    BOOL bMacroRunning;

    //! scheinbar ueberfluessig
    BOOL bNewCurveNeeded;

    //! setzt Titel des AreaScanFensters
    //! Rueckkehrcode immer True
    BOOL SetTitle( void );

    //! Funktion speichert Headerinformationen zu einem Areascan in einer Datei
    //! 1.Parameter gibt an, ob die Datei neu erstellt(=0) werden soll oder nur
    //! aktualisiert (ungl. 0) werden soll
    //! Rueckkehrcode false wenn probleme beim erstellen bzw. oeffnen der Datei
    //! sonst immer true
    BOOL SaveMeasurementInfo( BOOL );

    //! laden der Messungsinfos aus einer .psd datei
    //! hFile ist Filehandle der Messdatei
    //! Rueckkehrcode False wenn Datei zu kurz ,[HEADER] nicht vorhanden,
    //! Filetyp ungl. 1 oder OmegaStep = 0
    //! true sonst
    BOOL LoadMeasurementInfo( HFILE );

    //! zeigt an, ob gerade das erste Spektrum eines AreaScans aktiv ist
    //! wird benutzt, da die Datenbasis bereits mit einer Kurve init. wird
    //! aber in den folgendenSchritten immer eine neue Kurve anlegen muss
    BOOL bFirstScan;

    //! Kurve enthaelt zu jedem Spektrum der Datenbasis die Omega-,Thetapos.
    //! und einen Reportwert
    LPCurve ScanReport;


    //! abhaengig von eSaveFormat
    //! Rueckkehrcode bei ShiftedStandard ist die Anzahl der Psd-kanaele die
    //! benoetigt werden um den Thetabereich
    //! zu ueberdecken
    //! sonst immer 0
    int GetAdditionalColumns( void );

    //! Funktion liefert zu Anzahl der Scans (1.Parameter) Anzahl der
    //! Psd-kanaele, die benoetigt werden um den Thetabereich zu ueberdecken
    //! (nur bei ShiftedStandard)
    //! sonst 0 als Rueckkehrcode
    int GetShift( int ScanNumber );

    //! soll AreaScanstart vor Aufruf einer Einstellungsbox verhindern
    //! FIX Fehler 2
    BOOL bScanSetupOk;
};

//*****************************************************************************
class TChooseScan : public TModelessDlg
{
public:
    TChooseScan( TAreaScan* );

private:

    //! setzt Defaultwerte der Dialogbox bei Aufruf der Box
    //! Rueckkehrcode immer true
    BOOL Dlg_OnInit( HWND, HWND, LPARAM );

    //! behandelt Kommandos in der Dialogbox
    void Dlg_OnCommand( HWND, int, HWND, UINT );

    //! wird aufgerufen, wenn Scrollbar angeclickt wurde
    void Dlg_OnVScrollBar( HWND, HWND, UINT, int );

    //! wird aufgerufen, wenn in der Dialogbox die linke Maustaste nach
    //! Druecken wieder losgelassen wird (sinnvoll wenn Dialogbox waehrend
    //! eines Areascans aktiv ist)
    void Dlg_OnLButtonUp( HWND, int, int, UINT );

    //! wird aufgerufen beim druecken des OK-Buttons
    //! Rueckkehrcode immer true
    BOOL CanClose( void );

    //! Pointer auf das zugehoerige Areascanfenster
    TAreaScan* AScan;

    //! Pointer auf akt. angezeigte Kurve
    TCurve* Curve;

    //! Pointer auf Scanreportkurve der datenbasis
    TCurve* Report;

    //! Handle des Scrollbars
    HWND BarHandle;

    //! enthaelt Nr. des letzten Scans in Datenbasis
    int nMaxScan;
};

class TSetupAreaScan : public TModalDlg
{
public:
    TSetupAreaScan( TAreaScan* );

private:

    //! enthaelt Nummer des Omega- sowie Thetamotors in der Motorenliste
    int nOmega, nTheta,

    //! scheinbar ueberfluessig
    nOldMotor;

    //! gibt ausgewaehlten Detektor an
    LPDevice Device;

    //! Pointer auf AreaScanfenster
    TAreaScan* Scan;

    //! Handle der Detektorauswahlbox
    HWND hDeviceList;

    //! realisiert test der Dialogboxinhalte auf ihre konsistenz vor verlassen
    //! der Dialogbox (durch OK-Button)
    //! Rueckkehrcode false wenn die Dialogbox nicht geschlossen werden kann
    //! (Werte inkonsistent) sonst true
    BOOL CanClose( void );

    //! behandelt Kommandos in der Dialogbox
    void Dlg_OnCommand( HWND, int, HWND, UINT );

    //! setzt Defaultwerte der Dialogbox bei Aufruf der Box
    //! Rueckkehrcode immer true
    BOOL Dlg_OnInit( HWND, HWND, LPARAM );

    //! wird bei verlassen der Dlg.box aufgerufen (in dieser Dlg.box keine
    //! Bedeutung) da auch geerbt
    BOOL LeaveDialog( void );
};

class TAquisition : public TModalDlg
{
public:
    TAquisition( void );

private:

    //! scheinbar ueberfluessig
    int nNumberPoints;

    //! scheinbar ueberfluessig
    int nRange;

    //! scheinbar ueberfluessig
    char FileName[ 20 ];

    //! scheinbar ueberfluessig
    HWND hRangeList;

    //! realisiert test der Dialogboxinhalte auf ihre konsistenz vor verlassen
    //! der Dialogbox (durch OK-Button)
    //! Rueckkehrcode false wenn die Dialogbox nicht geschlossen werden kann
    //! (Werte inkonsistent) sonst true
    BOOL CanClose( void );

    //! hier werden IDOK und IDCANCEL behandelt
    void Dlg_OnCommand( HWND, int, HWND, UINT );

    //! eintragen von Defaultwerten fuer alle Elemente bei Aufruf der Dlg.Box
    //! rueckkehrcode immer true
    BOOL Dlg_OnInit( HWND, HWND, LPARAM );

    //! wird bei verlassen der Dlg.box aufgerufen (in dieser Dlg.box keine
    //! Bedeutung) da auch geerbt
    BOOL LeaveDialog( void );
};

class TSetupContinuousScan : public TModalDlg
{
public:
    TSetupContinuousScan( TScan* );

private:
    //! handles auf motor/detektor-comboboxen
    HWND hDeviceList, hMotorList;

    int //! akt. Motor
    nMotor,
    //! ?? ueberfluessig
    nOldMotor;

    //! akt. Detektor
    TDevice* Device;

    //! zugehoeriges StepScanfenster
    TScan* Scan;

    //! realisiert test der Dialogboxinhalte auf ihre konsistenz vor verlassen
    //! der Dialogbox (durch OK-Button)
    //! Rueckkehrcode False wenn die Dlg.box nicht geschlossen werden kann
    //! (Werte inkonsistent) sonst true
    BOOL CanClose( void );

    //! behandelt alle kommandos der Dialogbox
    void Dlg_OnCommand( HWND, int, HWND, UINT );

    //! setzen der Defaultwerte der Dialogbox
    //! rueckkehrcode: immer true
    BOOL Dlg_OnInit( HWND, HWND, LPARAM );

    //! ?? ueberfluessig da vererbt
    //! rueckkehrcode: immer true
    BOOL LeaveDialog( void );

    double //! Wert Startpunkt in Dlg.box
    dArgumentMin,
    //! ?? ueberfluessig
    dArgumentMax,
    //! Wert Geschwindigkeit in Dlg.box
    dSpeed;

    //! ?? nicht verwendet
    int nMotorOld;

    //! ?? nicht verwendet
    LPDevice DeviceOld;

    //! ?? ueberfluessig
    DWORD dwMaxCounts;

    //! Wert Messpunkte in Dlg.box
    int nCollectionAmount;

    //! Wert Messzeit in Dlg.box
    float fMaxTime;

    //! ?? ueberfluessig
    BOOL bSaveOnReady;

    //! ?? ueberfluessig
    TSaveFormat eSaveFormat;

    //! ?? verwendet aber unsinnig
    TxScanType eScanType;
};

class TSetupStepScan : public TModalDlg
{
public:
    TSetupStepScan( TScan* );

private:
    //! handles auf motor/detektor-Comboboxen
    HWND hDeviceList;
    HWND hMotorList;

    //! handle auf Scantyp-combobox
    HWND hScanTypeList;

    int //! akt. Motor
    nMotor,
    //! ?? ueberfluessig
    nOldMotor;

    //! akt. detektoren fuer Messung und Monitor
    LPDevice Device, Monitor;

    //! zugehoeriges StepScanfenster
    TScan* Scan;

    //! realisiert test der Dialogboxinhalte auf ihre konsistenz vor verlassen
    //! der Dialogbox (durch OK-Button)
    //! Rueckkehrcode False wenn die Dlg.box nicht geschlossen werden kann
    //! (Werte inkonsistent) sonst true
    BOOL CanClose( void );

    //! behandelt alle kommandos der Dialogbox
    void Dlg_OnCommand( HWND, int, HWND, UINT );

    //! setzen der Defaultwerte der Dialogbox
    //! rueckkehrcode: immer true
    BOOL Dlg_OnInit( HWND, HWND, LPARAM );

    //! ?? ueberfluessig da vererbt
    //! rueckkehrcode: immer true
    BOOL LeaveDialog( void )
    {
        return TRUE;
    };

    double //! Wert Minimum in Dlg.box
    dArgumentMin,
    //! Wert Maximum in Dlg.box
    dArgumentMax,
    //! Wert Schrittweite in Dlg.box
    dArgumentWidth;

    //! ?? nicht verwendet
    int nMotorOld;

    //! ?? nicht verwendet
    LPDevice DeviceOld;

    //! ?? nicht verwendet
    LPDevice MonitorOld;

    //! Wert Impulse in Dlg.box
    DWORD dwMaxCounts;

    //! Wert Zeit in Dlg.box
    float fMaxTime;

    BOOL //! ?? scheinbar ueberfluessig
    bMonitor,
    //! Wert Bei Beenden speichern
    bSaveOnReady;

    //! ?? ueberfluessig
    TSaveFormat eSaveFormat;

    //! Wert Scan-Typ in Dlg.box
    TxScanType eScanType;

    //! Wert Staendig Sichern in Dlg.box
    BOOL bSaveContinuous;

    //! Wert Monitor-Signal in Dlg.box
    BOOL bMonitorUsed;

    //! Wert Sicherungsverzeichnis
    char szPath[ 2*MaxString ];
};

class TCalibratePsd : public TModalDlg
{
public:
    TCalibratePsd( TCurve& );

    //! ermittelt Anzahl der Scrollbarschritte links bzw. rechts von der
    //! absoluten Null
    //! 1. Parameter side==LEFT Anzahl der Scrollbarschritte links
    //! von der abs. Null
    //! sonst Anzahl der Scrollbarschritte rechts
    //! von der abs. Null
    //! Rueckkehrcode Anzahl der Scrollbarschritte links bzw. rechts
    //! von der abs. Null
    //! Achtung diese Funktion muss mit LEFT zuerst aufgerufen werden
    int GetBarEgde( int );

    //! liefert die Position des Scrollbar anhand der akt. Motorposition im
    //! Rueckkehrcode
    int GetBarPos( void );

    //! behandelt alle Scrollbarereignisse
    void Dlg_OnHScrollBar( HWND, HWND, UINT, int );

    //! setzt Defaultwerte der Dialogbox bei Aufruf der Box
    //! Rueckkehrcode immer true
    BOOL Dlg_OnInit( HWND, HWND, LPARAM );

    //! behandelt Timersignale zur aktualisierung der Winkelwerte in Dialogbox
    void Dlg_OnTimer( HWND, UINT );

    //! behandelt Kommandos in der Dialogbox
    void Dlg_OnCommand( HWND, int, HWND, UINT );

    //! wird aufgerufen beim druecken des Beenden-Buttons
    //! Rueckkehrcode False wenn der Winkel pro Kanal ==0
    //! sonst true
    BOOL CanClose( void );

    //! wird bei verlassen der dialogbox aufgerufen Rueckkehrcode immer true
    BOOL LeaveDialog( void );

private:

    //! Zeitintervall zur akt. der Winkel in Dialogbox
    int nAskTime;

    //! gibt an ob motor in bewegung
    BOOL bMoveActive;

    //! gibt umrechnungsfactor an um Winkelsekunden in die jeweilige Einheit
    //! umzurechnen
    float fRelationFactor;

    //! wahrscheinlich ueberfluessig
    BOOL bCalibrationOk;

    //! akt. angezeigte Kurve
    LPCurve Curve;

    //! gibt pos. des linken bzw. rechten Peaks an
    CPoint LeftPeak;
    CPoint RightPeak;

    //! gibt an ob rechter bzw. linker Peak schon erreicht ist
    BOOL bGetRightPeak;
    BOOL bGetLeftPeak;

    //! gibt an ob motor ueber groessere distanz faehrt
    BOOL bLongMove;

    //! scheinbar ueberfluessig
    float fOldPsdStep;

    //! FIX Fehler 49
    //! beinhaltet die Schrittweite vor Aufruf der Calibr.-Box 
    double dOldMotorWidth;

    //! Handle auf Dialogelement
    HWND hDlgItem;

    //! Handle des Scrollbars
    HWND hBar;

    //! gibt an ob Abzahl der Scrollbarschritte kleiner als 20000 ist (1) oder
    //! nicht (2)
    int BarFactor;

    //! enthaelt Nummer des Thetamotors in der Motorenliste
    int nMotor;

    //! FIX Fehler 48
    //! Wert fuer Winkel pro Kanal 
    float dStepWidth;

    //! Pointer auf einen Psd
    TDevice* Device;
};

//! diese Dialogbox funktioniert nicht(unvollstaendig)
class TPsdRemoteSync : public TModalDlg
{
public:
    TPsdRemoteSync( TCurve& );
    BOOL Dlg_OnInit( HWND, HWND, LPARAM );
    void Dlg_OnTimer( HWND, UINT );
    void Dlg_OnCommand( HWND, int, HWND, UINT );
    BOOL CanClose( void );
    BOOL LeaveDialog( void );

private:
    void PrintFileName( void );
    LPCSTR GetFileName( void );
    int nAskTime;
    BOOL bCalibrationOk;
    LPCurve Curve;
    float fOldPsdStep;
    HWND hDlgItem;
    TDevice* Device;
    int nCurveCount;
    char aPath[ 50 ];
    char aName[ 14 ];
    float fExposureTime;
    DWORD dwExposureCounts;
    float fError;
    char aFileName[ 127 ];
};

#endif
// __M_XSCAN_H
// __LastLine__

