#include "testdev.h"

long _huge testDevData[9][17][601];


Testdev::Testdev() {
  TDevice();
  scanFile = fopen( "testdev.dat", "r" );
  if (!scanFile) {
    MessageBox( NULL, "testdev.dat fehlt", "Fehler", MBSTOP );
    PostQuitMessage( 0 );
  }
  else
    if (!( fillDevData() )) {
      MessageBox( NULL, "testdev.dat fehlerhaft", "Fehler", MBSTOP );
      PostQuitMessage( 0 );
    }
}


int Testdev::PollDevice( void ) {
  if(!bDeviceActive)
    return R_Failure;
  KillTimer(hWndFrame,DeviceTimerIdStart + GetId());
  if(bDebug)
    SetInfo("MPoll");
  fIntensity = CalculateIntensity();
  if (fIntensity == 0.0)
    fSigma = 1.0;
  else
    fSigma = sqrt( fIntensity ) * 0.9 / fIntensity;
  bDataValid = TRUE;
  if (hControlWnd)
    PostMessage( hControlWnd, WM_COMMAND, wpJumpTo, GetId() );
  else
    if (hDisplayWnd)
      PostMessage( hDisplayWnd, WM_COMMAND, wpJumpTo, GetId() );
  bDeviceActive = FALSE;
  return R_MeasOk;
}


double Testdev::CalculateIntensity() {
  double actDF, actTL, actCC, retValue;
  int DFIndex, TLIndex, CCIndex;
  int TLBias;
  // Aktuelle Motorpositionen ermitteln
  mlGetDistance( mlGetIdByName( Omega ), actDF );
  mlGetDistance( mlGetIdByName( Psi ), actTL );
  mlGetDistance( mlGetIdByName( Collimator ), actCC );
  // Auf gerade Werte runden
  (actDF < 0)
    ? (DFIndex = (int)(actDF - 0.5))
    : (DFIndex = (int)(actDF + 0.5));
  (actTL < 0)
    ? (TLIndex = (int)(actTL - 0.5))
    : (TLIndex = (int)(actTL + 0.5));
  (actCC < 0)
    ? (CCIndex = (int)(actCC - 0.5))
    : (CCIndex = (int)(actCC + 0.5));
  // Eingrenzung auf das vermessene Gebiet (s. testdev.dat)
  if (DFIndex < -300)
    DFIndex = -300;
  if (DFIndex > 300)
    DFIndex = 300;
  // Umsetzung des Winkelwerts in den entsprechenden Feldindex
  DFIndex = DFIndex + 300;
  // Eingrenzung auf das vermessene Gebiet (s. testdev.dat)
  if (TLIndex < -40)
    TLIndex = -40;
  if (TLIndex > 40)
    TLIndex = 40;
  // Vorbereitung der Interpolation
  TLBias = (TLIndex + 40) % 5;
  // Umsetzung des Winkelwerts in den entsprechenden Feldindex
  TLIndex = (TLIndex + 40)/5;
  // Eingrenzung auf das vermessene Gebiet (s. testdev.dat)
  if (CCIndex < -280)
    CCIndex = -280;
  if (CCIndex > 280)
    CCIndex = 280;
  // Umsetzung des Winkelwerts in den entsprechenden Feldindex
  CCIndex = (CCIndex + 280)/70;
  // Auslesen und u.U. Interpolation der korrespondierenden Intensitt
  if ( TLBias )
    retValue = InterPolate( CCIndex, TLIndex, DFIndex,  TLBias );
  else
    retValue = testDevData[CCIndex][TLIndex][DFIndex];
  // Simulation "zufaelliger" Schwankungen (Messfehler)
  retValue += (rand()%10 - 5);

  return retValue;
}


  double Testdev::InterPolate( int ccI, int tlI, int dfI, int tlB ) {
  double mVal, rVal, lVal;

  lVal = testDevData[ccI][tlI][dfI];
  rVal = testDevData[ccI][tlI + 1][dfI];
  mVal = ((5-tlB)*lVal + tlB*rVal)/5.0;
  return mVal;
}


BOOL Testdev::fillDevData() {
  char actLine[512];
  char temp[20];
  int x, y, z;
  double fval;

  if (!scanFile)
    return FALSE;
  for (x = 0; x < 9; x++) {
    if (!(fgets( actLine, 512, scanFile )) ||
        strcmp( strtok( actLine, " " ), "C" ))
    {
      MessageBox( NULL, "Collimator - \"C\" expected", "Syntax Error", MB_ICONSTOP );
      return FALSE;
    }
    for (y = 0; y < 17; y++) {
      if (!(fgets( actLine, 512, scanFile )) ||
          strcmp( strtok( actLine, " " ), "T" ))
      {
        MessageBox( NULL, "Tilt - \"T\" expected", "Syntax Error", MB_ICONSTOP );
        return FALSE;
      }
      for (z = 0; z < 601; z++ ) {
        if (!(fgets( actLine, 512, scanFile )) ||
            strcmp( strtok( actLine, " " ), "D" ))
        {
          MessageBox( NULL, "Diffraction - \"D\" expected", "Syntax Error", MB_ICONSTOP );
          return FALSE;
        }
        if (!(fgets( actLine, 512, scanFile )) ||
            strcmp( strtok( actLine, " " ), "I" ))
        {
          MessageBox( NULL, "Intensity - \"I\" expected", "Syntax Error", MB_ICONSTOP );
          return FALSE;
        }
        strcpy(temp, strtok( NULL, "\n" ));
        fval = (atof( temp ));
        testDevData[x][y][z] = (long)((fval) + 0.5);
      }
    }
  }
  return TRUE;
}

